/**
 * Copyright 2019-2023 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ Shell (https://gitlab.com/DesQ/Shell)
 * DesQ Shell is the main Shell UI of the DesQ Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#include <QDebug>
#include <QtDBus>
#include <QLabel>

#include "Global.hpp"
#include "TasksHost.hpp"
#include "Task.hpp"

#include <wayqt/Registry.hpp>
#include <wayqt/DesQShell.hpp>
#include <wayqt/WayQtUtils.hpp>
#include <wayqt/ToplevelManager.hpp>

#include <DFXdg.hpp>

QIcon iconForAppId( QString appId ) {
    /** appId == name of the desktop file */
    DFL::XDG::DesktopFile desktop( appId );

    /**
     * In case the appId does not amtch the desktop name,
     * let's check the lower case version of it
     */
    if ( desktop.isValid() == false ) {
        desktop = DFL::XDG::DesktopFile( appId.toLower() );
    }

    QString icon = desktop.icon();

    if ( icon.length() and QIcon::hasThemeIcon( icon ) ) {
        return QIcon::fromTheme( icon );
    }

    /** Check the app id directly! */
    if ( QIcon::hasThemeIcon( appId ) ) {
        return QIcon::fromTheme( appId );
    }

    /** Check the lower case version of app id directly */
    if ( QIcon::hasThemeIcon( appId.toLower() ) ) {
        return QIcon::fromTheme( appId.toLower() );
    }

    /** Not worknig hard: Just return application-x-executable */
    return QIcon::fromTheme( "application-x-executable" );
}


TasksPluginWidget::TasksPluginWidget( QWidget *parent ) : DesQ::Panel::PluginWidget( parent ) {
    tasksLayout = new QHBoxLayout();
    tasksLayout->setContentsMargins( QMargins() );
    tasksLayout->setSpacing( 5 );

    setLayout( tasksLayout );

    if ( wlRegistry->waitForInterface( WQt::Registry::DesQShellInterface ) ) {
        shell = wlRegistry->desqShell();
        // connect(
        //     this, &TasksPluginWidget::panelScreenChanged, [ = ] ( QScreen *scrn ) {
        //         output = shell->getOutput( WQt::Utils::wlOutputFromQScreen( scrn ) );
        //         connect( output, &WQt::DesQOutput::viewAdded, this, &TasksPluginWidget::addTask );
        //         output->setup();
        //     }
        // );
    }

    if ( wlRegistry->waitForInterface( WQt::Registry::ToplevelManagerInterface ) ) {
        tlm = wlRegistry->toplevelManager();
        connect(
            tlm, &WQt::ToplevelManager::newToplevel, [ = ] ( WQt::Toplevel *tl ) {
                addTask( tl );
            }
        );

        connect(
            tlm, &WQt::ToplevelManager::toplevel, [ = ] ( WQt::Toplevel *tl ) {
                addTask( tl );
            }
        );

        tlm->setup();
    }
}


TasksPluginWidget::~TasksPluginWidget() {
    delete tlm;
}


void TasksPluginWidget::addTask( WQt::Toplevel *tl ) {
    if ( idToplevelMap.contains( tl->uuid() ) ) {
        return;
    }

    idTaskMap[ tl->uuid() ] = new TaskButton( this );
    idTaskMap[ tl->uuid() ]->setIcon( iconForAppId( tl->appId() ) );
    idTaskMap[ tl->uuid() ]->setToolTip( tl->title() );
    idTaskMap[ tl->uuid() ]->setFixedSize( QSize( panelHeight - 2, panelHeight - 2 ) );
    idTaskMap[ tl->uuid() ]->addTask( tl );

    connect(
        tl, &WQt::Toplevel::appIdChanged, [ = ] ( QString, QString ) {
            idTaskMap[ tl->uuid() ]->setIcon( iconForAppId( tl->appId() ) );
        }
    );

    connect(
        tl, &WQt::Toplevel::titleChanged, [ = ] ( QString, QString newTitle ) {
            idTaskMap[ tl->uuid() ]->setToolTip( newTitle );
        }
    );

    connect(
        tl, &WQt::Toplevel::stateChanged, [ = ] ( uint32_t, uint32_t ) {
            if ( tl->isActive() ) {
                idTaskMap[ tl->uuid() ]->setChecked( true );
            }

            else {
                idTaskMap[ tl->uuid() ]->setChecked( false );
            }
        }
    );

    connect(
        tl, &WQt::Toplevel::closed, [ = ] () {
            removeTask( tl );
        }
    );

    tasksLayout->addWidget( idTaskMap[ tl->uuid() ] );

    tl->setup();
}


void TasksPluginWidget::removeTask( WQt::Toplevel *tl ) {
    uint32_t id = tl->uuid();

    idToplevelMap.remove( id );
    QToolButton *btn = idTaskMap.take( id );

    delete btn;
    delete tl;
}
